#
# Copyright 2008 VMware, Inc.  All rights reserved.
#

import sys
import xf86config

def getOption(options, name):
   """ Returns the option entry if the given options has an entry with
   name, otherwise None"""
   for o in options:
      if o.name == name:
         return o

   return None

def install(driver):
   def configureMouse(config, mouse):
      """ Configure vmmouse """
      mouse.driver = 'vmmouse'

      options = { 'Buttons'         : '5',
                  'ZAxisMapping'    : '4 5',
                  'Emulate3Buttons' : 'true',
                  'Protocol'        : 'IMPS/2',
                  'CorePointer'     : None,
                  'Device'          : '/dev/input/mice' }

      for option, value in options.items():
         optEntry = getOption(mouse.options, option)
         if not optEntry:
            mouse.options.insert(xf86config.XF86Option(option, value))
         else:
            optEntry.val = value

   def configureDisplayDevice(config, device):
      device.driver = 'vmware'

   # Returns a unique input device identifier.
   def getUniqueInputDevice(config):
      for i in range(50):
         ident = 'Mouse%d' % i

         try:
            xf86config.lookupInputDevice(config, ident)
         except xf86config.XF86SectionMissing:
            return ident
      else:
         raise Exception('Unable to generate a unique mouse identifier')

   if driver == 'vmmouse':
      try:
         pointer = xf86config.getCorePointer(config)
      except xf86config.XF86SectionMissing:
         # RHEL 5 doesn't come with a mouse section so we have to add
         # it if it's missing.
         pointer = xf86config.XF86ConfInput()
         config.input.insert(pointer)
         pointer.identifier = getUniqueInputDevice(config)
         config.layout[0].inputs.insert(xf86config.XF86ConfInputref(
               pointer.identifier, 'CorePointer'))

      configureMouse(config, pointer)

   elif driver == 'vmware':
      screen = xf86config.getPrimaryScreen(config)
      configureDisplayDevice(config, xf86config.lookupDevice(config, screen.device))
   else:
      raise ValueError('driver must be either vmware or vmmouse')

def uninstall(driver):
   def configureMouse(config, mouse):
      mouse.driver = 'mouse'

   def configureDisplayDevice(config, device):
      device.driver = 'vesa'

   if driver == 'vmmouse':
      configureMouse(config, xf86config.getCorePointer(config))
   elif driver == 'vmware':
      screen = xf86config.getPrimaryScreen(config)
      configureDisplayDevice(config, xf86config.lookupDevice(config, screen.device))
   else:
      raise ValueError('driver must be either vmware or vmmouse')


def printUsage(program):
   print 'Usage: %s (install|uninstall) (vmmouse|vmware)' % program

config, configPath = xf86config.readConfigFile()

if not config:
   raise Exception('Unable to read X config file')

if len(sys.argv) == 3:
   cmd, driver = sys.argv[1:]

   if cmd == 'install':
      install(driver)
   elif cmd == 'uninstall':
      uninstall(driver)
   else:
      printUsage(sys.argv[1])
else:
   printUsage(sys.argv[0])
   sys.exit(1)

config.write(configPath)
